"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var ConfigService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigService = void 0;
const class_validator_1 = require("class-validator");
const class_transformer_1 = require("class-transformer");
const common_1 = require("@nestjs/common");
const errors_1 = require("./errors");
const exceptions_1 = require("./exceptions");
const utils_1 = require("./utils");
let ConfigService = ConfigService_1 = class ConfigService {
    configs;
    envs;
    logger = new common_1.Logger(ConfigService_1.name);
    constructor(configs, envs) {
        this.configs = configs;
        this.envs = envs;
    }
    add(providers) {
        providers.forEach(provider => {
            if (!this.configs.has(provider)) {
                const instance = (0, class_transformer_1.plainToInstance)(provider, this.envs, {
                    enableImplicitConversion: true,
                    exposeDefaultValues: true
                });
                this.configs.set(provider, instance);
            }
        });
        return this;
    }
    get(provider) {
        const config = this.configs.get(provider);
        if (!config) {
            throw new exceptions_1.ConfigServiceException(`${errors_1.ConfigServiceError.CONFIG_NOT_FOUND}: [class ${provider.name}]`);
        }
        return config;
    }
    value(provider, key, defaultValue) {
        const config = this.configs.get(provider);
        if (!config) {
            throw new exceptions_1.ConfigServiceException(`${errors_1.ConfigServiceError.CONFIG_NOT_FOUND}: [class ${provider.name}]`);
        }
        return defaultValue || config[key];
    }
    values(...providers) {
        return providers.reduce((values, provider) => {
            const config = this.configs.get(provider);
            if (!config) {
                throw new exceptions_1.ConfigServiceException(`${errors_1.ConfigServiceError.CONFIG_NOT_FOUND}: [class ${provider.name}]`);
            }
            const properties = (0, utils_1.getInstanceProperties)(config);
            return {
                ...values,
                ...properties
            };
        }, {});
    }
    override(provider, overrides) {
        const config = this.configs.get(provider);
        if (!config) {
            throw new exceptions_1.ConfigServiceException(`${errors_1.ConfigServiceError.CONFIG_NOT_FOUND}: [class ${provider.name}]`);
        }
        return Object.assign(config, overrides || {});
    }
    validate(providers) {
        const validations = providers.map(provider => {
            const config = this.configs.get(provider);
            if (!config) {
                throw new exceptions_1.ConfigServiceException(`${errors_1.ConfigServiceError.CONFIG_NOT_FOUND}: [class ${provider.name}]`);
            }
            return {
                provider,
                errors: (0, class_validator_1.validateSync)(config)
            };
        });
        if (validations.some(config => config.errors.length)) {
            validations.forEach(config => {
                if (!config.errors.length) {
                    return;
                }
                this.logger.error(`Failed to validate config [class ${config.provider.name}]:`);
                config.errors.forEach(error => Object.entries(error.constraints || {}).forEach(([_, message]) => {
                    this.logger.error(`\x1b[31m└─ \x1b[90m${message} (found: ${error.value})\x1b[0m`);
                }));
            });
            throw new exceptions_1.ConfigServiceException(errors_1.ConfigServiceError.CONFIG_VALIDATION_ERRORS);
        }
    }
    getConfigs() {
        return this.configs;
    }
};
exports.ConfigService = ConfigService;
exports.ConfigService = ConfigService = ConfigService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)('GLOBAL_CONFIG_MAP')),
    __param(1, (0, common_1.Inject)('GLOBAL_ENVIRONMENT_VARIABLES')),
    __metadata("design:paramtypes", [Object, Object])
], ConfigService);
//# sourceMappingURL=config.service.js.map