"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstanceProperties = exports.getEnvironmentVariables = void 0;
const dotenv_1 = require("dotenv");
(0, dotenv_1.config)();
const getEnvironmentVariables = () => process.env;
exports.getEnvironmentVariables = getEnvironmentVariables;
const getInstanceProperties = (instance) => {
    const properties = Object.getOwnPropertyNames(instance);
    const prototype = Object.getPrototypeOf(instance);
    const methods = Object.getOwnPropertyNames(prototype)
        .filter(name => name !== 'constructor');
    const propertyNames = [...methods, ...properties];
    return propertyNames.reduce((result, propertyName) => {
        const value = instance[propertyName];
        if (typeof value === 'function') {
            return {
                ...result,
                [propertyName]: (...args) => value.apply(dotenv_1.config, args)
            };
        }
        return {
            ...result,
            [propertyName]: value
        };
    }, {});
};
exports.getInstanceProperties = getInstanceProperties;
//# sourceMappingURL=utils.js.map